/**
 * Metrics service is a set of utility classes to enable a Java
 * programmer to send metrics to the Zabbix monitoring tool.
 * 
 * Copyright 2007, Gerco Dries (gdr@progaia-rs.nl).
 * 
 * Metrics service is free software: you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * Metrics service is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Metrics service.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.progaia.metricsservice;

import com.sonicsw.mf.common.metrics.*;
import com.sonicsw.mf.common.metrics.manager.*;
import com.sonicsw.mf.common.metrics.IMetricIdentity;
import com.sonicsw.xqimpl.service.XQContainer;

/**
 * 
 * The Metric class implements a simple interface to a Sonic framework metric. Simply
 * create a Metric object and call it's increment() method when you want to change the 
 * value. The Metric class takes care of publishing and registering the Metric with Sonic
 * 
 * Threading note: This class exposes the 'this' pointer in the constructor. Therefore 
 * thread-safety is limited to the setValue(), getValue() and increment() methods.
 * 
 * @author Gerco Dries (gdr@progaia-rs.nl)
 *
 */
public class Metric implements IStatisticProvider {
	private final IMetricIdentity identity;
	private final IMetricInfo     metric;
	private final IStatistic      statistic;
	private       long            value;
	
	/**
	 * Create a dynamic, non-instance metric without alerts support
	 * 
	 * @param name The metric name (eg. NumberOfRequests)
	 * @param description The metric description (eg. Number of requests processed since restart)
	 * @param unit The unit of measurement (eg. Requests, Messages, etc)
	 */
	public Metric(String name, String description, String unit) {
		// Create a dynamic, non-instance metric that doesn't support alerts
		this.identity = MetricsFactory.createMetricIdentity(name);		
		this.metric = MetricsFactory.createMetricInfo(this.identity, IValueType.VALUE, description, null, false, true, false, false, unit);
		this.statistic = StatisticsFactory.createStatistic((short)0, false, new IStatisticProvider[] {this}, (short)0);
		
		setValue(0);
		
		// Register the metric with the ESB Container
		XQContainer.registerMetricInfo(this.metric, this.statistic);
	}
	
	private synchronized void setValue(long value) {
		this.value = value;
	}

	private synchronized long getValue() {
		return value;
	}
	
	/**
	 * Increment the value of the metric
	 */
	synchronized void increment() {
		this.value++;
	}

	public void updateStatistic(ISampledStatistic s) {
		s.updateValue(getValue());
	}

	public void resetStatistic(ISampledStatistic s) {
		setValue(0);
	}
}
