/**
 * Metrics service is a set of utility classes to enable a Java
 * programmer to send metrics to the Zabbix monitoring tool.
 * 
 * Copyright 2007, Gerco Dries (gdr@progaia-rs.nl).
 * 
 * Metrics service is free software: you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * Metrics service is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Metrics service.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.progaia.metricsservice;
 
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;
import org.apache.xpath.objects.XObject;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
 
import com.sonicsw.xq.*;

/**
 * The MetricsService allows an ESB engineer to publish custom metrics. Currently, this is
 * limited to simple counter metrics, but it can be expanded to publish per second/minute/hour
 * counters as well.
 * 
 * @author Gerco Dries (gdr@progaia-rs.nl)
 */
public class MetricsService implements XQService {

	private XQLog m_xqLog;
    private Map m_Metrics;

    /**
     * Constructor for a MetricsService
     */
	public MetricsService () {
	}
	
	/**
     * Initialize the XQService by processing its initialization parameters.
     *
     * <p> This method implements a required XQService method.
     *
     * @param initialContext The Initial Service Context provides access to:<br>
     * <ul>
     * <li>The configuration parameters for this instance of the MetricsService.</li>
     * <li>The XQLog for this instance of the MetricsService.</li>
     * </ul>
     * @exception XQServiceException Used in the event of some error.
     */
    public void init(XQInitContext initialContext) throws XQServiceException {
    	m_xqLog = new PrefixingLogger("[MetricsService] ", initialContext.getLog());
    	
    	// Read metrics from config file and create them
    	createMetrics(initialContext.getParameters().getParameter("ConfigFile", XQConstants.PARAM_STRING));
    }


    private void createMetrics(String xml) {
    	try {
    		// Initialize the metrics map
			m_Metrics = new HashMap();
			
	    	DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	    	DocumentBuilder db = dbf.newDocumentBuilder();
	    	InputSource is = new InputSource(new StringReader(xml));
	    	Document d = db.parse(is);
			
	    	// For each <metric>
	    	NodeList nl = XPathAPI.selectNodeList(d.getDocumentElement(), "/metrics/metric");
	    	for(int i = 0; i<nl.getLength(); i++) {
	    		Node n = nl.item(i);
	    		String metricName = getXPathValue(n, "@id");
	    		String metricDescription = getXPathValue(n, "description");
	    		String metricUnit = getXPathValue(n, "unit");
	    		
	    		m_Metrics.put(metricName, new Metric(metricName, metricDescription, metricUnit));
	    		m_xqLog.logInformation("Registered metric " + metricName);
	    	}
	    	
	    	// Lock the metrics Map, to make sure no more modifications are made
	    	m_Metrics = java.util.Collections.unmodifiableMap(m_Metrics);
    	} catch (ParserConfigurationException e) {
    		e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}
    	finally {
    		m_xqLog.logInformation("Initialized");
    	}
	}

	/**
     * Handle the arrival of XQMessages in the INBOX.
     *
     * <p> This method implement a required XQService method.
     *
     * @param ctx The service context.
     * @exception XQServiceException Thrown in the event of a processing error.
     */
    public void service(XQServiceContext ctx) throws XQServiceException {
        if (ctx == null)
            throw new XQServiceException("Service Context cannot be null.");
        else {        	
            // Get the message.
            XQEnvelope env = null;
            while (ctx.hasNextIncoming()) {
                env = ctx.getNextIncoming();
                if (env != null) {
                	// Increase the metric value
                	XQParameters params = ctx.getParameters();
                	String metricID = params.getParameter("MetricID", XQConstants.PARAM_STRING);
                	Metric m = getMetric(metricID);
                	
                	if(m != null) {
                		m.increment();
                	}
                	
                    // Pass message onto the outbox.
                    Iterator addressList = env.getAddresses();
                    if (addressList.hasNext())  {
                        // Add the message to the Outbox
                        ctx.addOutgoing(env);
                    }
                }
            }
        }
    }
    
    private Metric getMetric(String metricName) {
    	return (Metric)m_Metrics.get(metricName);
	}

	/**
     * Clean up and get ready to destroy the service.
     *
     * <p> This method implement a required XQService method.
     */
    public void destroy() {
    }
    
	public static String getXPathValue(final Node context, final String xpath) {
		try {
			XObject resultObject = XPathAPI.eval(context, xpath);
			return resultObject == null ? "" : resultObject.str();
		} catch (TransformerException e) {
			throw new RuntimeException("Error while evaluating XPath expression '" + xpath + "': " + e.getClass().getName() + ": " + e.getMessage(), e);
		} catch (RuntimeException e) {
			throw new RuntimeException("Error while evaluating XPath expression '" + xpath + "': " + e.getClass().getName() + ": " + e.getMessage(), e);
		}
	}    
}